#
# Copyright (c) 1997-1998 Picture Elements, Inc.
# Stephen Williams (steve@picturel.com)
#
#    This source code is free software; you can redistribute it
#    and/or modify it in source code form under the terms of the GNU
#    Library General Public License as published by the Free Software
#    Foundation; either version 2 of the License, or (at your option)
#    any later version. In order to redistribute the software in
#    binary form, you will need a Picture Elements Binary Software
#    License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Library General Public License for more details.
#
#    You should have received a copy of the GNU Library General Public
#    License along with this program; if not, write to the Free
#    Software Foundation, Inc.,
#    59 Temple Place - Suite 330
#    Boston, MA 02111-1307, USA
#
#    You should also have recieved a copy of the Picture Elements
#    Binary Software License offer along with the source. This offer
#    allows you to obtain the right to redistribute the software in
#    binary (compiled) form. If you have not received it, contact
#    Picture Elements, Inc.,
#    777 Panoramic Way
#    Berkeley, CA 94704.
#
#ident "$Id: ikit_func.tcl,v 1.37 1999/02/10 03:59:57 steve Exp $"
#


# --
# This file adds a bunch of function selections to the ikit
# program. This package uses core capabilities to implement the
# various functions described herin.
#

ik_add_function "Laplacian" \
	laplacian
ik_add_function "Rotate~Clockwise 90"         {func_rotate +90}
ik_add_function "Rotate~Counter-clockwise 90" {func_rotate -90}
ik_add_function "Color to Gray~Y = 0.3R + 0.6G + 0.1B" \
	{func_rgb_to_y}
ik_add_function "Gray to Binary~Error Diffusion~Floyd-Steinberg" \
	{func_diffuse floyd-steinberg}
ik_add_function "Gray to Binary~Error Diffusion~Jarvis, Judice and Ninke" \
	{func_diffuse jjn}
ik_add_function "Gray to Binary~Error Diffusion~Stucki" \
	{func_diffuse stucki}
ik_add_function "Gray to Binary~Global Threshold" \
	func_threshold
ik_add_function "Gray to Binary~Classical Screen~M = 3" {func_halftone fh_M3}
ik_add_function "Scale~Bilinear 2 x 2"  {func_bilinear}
ik_add_function "Scale~Replicate~8 x 8" {func_replicate 8 8}
ik_add_function "Scale~Replicate~2 x 2" {func_replicate 2 2}
ik_add_function "Scale~Replicate~2 x 1" {func_replicate 2 1}
ik_add_function "Scale~Replicate~1 x 2" {func_replicate 1 2}
ik_add_function "Scale~Replicate~select" {func_replicate_interact}
ik_add_function "Fourier transform" func_fourier
ik_add_function "Filter~Smooth" \
	smooth
ik_add_function "Filter~High-pass" \
	func_hi_filter
ik_add_function "Filter~Detail Enhance" \
	func_detail_filter
ik_add_function "Compare Images~|a - b|" \
	func_compare
ik_add_function "Compare Images~(a - b)^2" \
	func_sqe
ik_add_function "Compare Colors~Projection" \
	func_rgbproj
ik_add_function "Compare Colors~Match" \
	func_rgbmatch
ik_add_function "Compare Colors~Project on white" \
	func_rgbproj_white
ik_add_function "Crop" func_crop
ik_add_function "Linear Contrast Expand" \
	func_cont_exp
ik_add_function "Composite Images~Fore/Alpha" \
	func_alpha_mask
ik_add_function "Composite Images~Fore/Alpha/Back" \
	func_alpha_back
ik_add_function "Plane~red (0)" \
	{func_plane 0}
ik_add_function "Plane~green (1)" \
	{func_plane 1}
ik_add_function "Plane~blue (2)" \
	{func_plane 2}
ik_add_function "Plane~alpha (3)" \
	{func_plane 3}
ik_add_function "Plane~select" \
	{func_plane_interact}
ik_add_function "Plane~Laminate planes" \
	{func_laminate}
ik_add_function "Quantize~Round to 8bits" \
	func_round
ik_add_function "Quantize~Hyper Median (color)" \
	func_quant
ik_add_function "Quantize~1d Median (gray)" \
	func_red1dmed
ik_add_function "Quantize~332 (RGB color)" \
	func_red332
ik_add_function "Quantize~3plane diffuse" \
	func_color_diffuse
ik_add_function "Synthesize~Solid color" \
	func_solid

ik_add_function "Plot~Column average" func_average_column

proc func_alpha_mask {} {
    set src1 [ik_stack_peek 0]
    set src2 [ik_stack_peek 1]
    if {[llength "$src1 $src2"] != 2} { return }

    set image [image create ik_image -alpha $src1 $src2]
    ik_stack_push $image "$image <-- alpha($src1, $src2)\n"
}

proc func_alpha_back {} {
    set src1 [ik_stack_peek 0]
    set src2 [ik_stack_peek 1]
    set src3 [ik_stack_peek 2]
    if {[llength "$src1 $src2 $src3"] != 3} { return }

    set image [image create ik_image -alpha $src1 $src2 $src3]
    ik_stack_push $image "$image <-- alpha($src1, $src2, $src3)\n"
}

# --
# The laplacian looks at an image on the top of the stack, calculates
# the laplacian of it, and pushes the result.

set laplacian_gain 10
proc laplacian {} {
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }
    global  laplacian_gain

    ik_get_values "{laplacian_gain {Output gain:} integer}"

    set image [image create ik_image -laplace $source -out-gain $laplacian_gain]

    ik_stack_push $image "$image <-- Laplace($source, $laplacian_gain)\n"
}

proc smooth {} {
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    set m(x) 3
    set m(y) 3
    set m(0,0) 1 ; set m(1,0) 1 ; set m(2,0) 1
    set m(0,1) 1 ; set m(1,1) 1 ; set m(2,1) 1
    set m(0,2) 1 ; set m(1,2) 1 ; set m(2,2) 1

    set image [image create ik_image -convolute $source -matrix m -gain 1/9]
    .body.console insert end "$image <-- smooth($source)\n"
    .body.stack.list insert 0 $image
}

proc func_rotate {dir} {
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    set image [image create ik_image -rotate $source $dir]
    ik_stack_push $image "$image <-- rotate($source, $dir)\n"
}

proc func_compare {} {
    set src1 [ik_stack_peek 0]
    if {[llength $src1] == 0} { return }

    set src2 [ik_stack_peek 1]
    if {[llength $src2] == 0} { return }

    set image [image create ik_image -compare $src1 $src2]
    ik_stack_push $image "$image <-- compare($src1, $src2)\n"
}

proc func_crop {} {
    global crop_x crop_y crop_w crop_h
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    set crop_x 0
    set crop_y 0
    set crop_w [ik_image $source width]
    set crop_h [ik_image $source height]

    ik_get_values {
	{crop_x "x" integer}
	{crop_y "y" integer}
	{crop_w "width" integer}
	{crop_h "height" integer}
    }

    set image [image create ik_image -crop $source -cut $crop_x $crop_y $crop_w $crop_h]
    ik_stack_push $image "$image <-- Crop($source, $crop_x $crop_y $crop_w $crop_h)\n"
}

set cont_exp(low) 0
set cont_exp(high) 255
proc func_cont_exp {} {
    global cont_exp
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    ik_get_values {
	{cont_exp(low) "low" integer}
	{cont_exp(high) "high" integer}
    }
	
    set image [image create ik_image -lcexp $source \
	    -low $cont_exp(low) -high $cont_exp(high)]
    ik_stack_push $image \
	    "$image <-- ContrExp($cont_exp(low), $cont_exp(high))\n"
}

set diffuse_prop 128
proc func_diffuse {filter} {
    global diffuse_prop
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    ik_get_values {
	{diffuse_prop "Propagation (0-128)" integer}
    }

    set image [image create ik_image -diffuse $source -filter $filter -propagate $diffuse_prop]
    ik_stack_push $image "$image <-- Diffuse($source, $filter, $diffuse_prop)\n"
}


proc func_fourier {} {
    set source [ik_stack_peek]
    if {[llength $source] == 0} {return}

    set image [image create ik_image -fourier $source]
    ik_stack_push $image "$image <-- fourier($source)\n" ""
}


proc func_hi_filter {} {
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    set m(x) 3
    set m(y) 3
    set m(0,0) -1 ; set m(1,0) -1 ; set m(2,0) -1
    set m(0,1) -1 ; set m(1,1)  8 ; set m(2,1) -1
    set m(0,2) -1 ; set m(1,2) -1 ; set m(2,2) -1

    set image [image create ik_image -convolute $source -matrix m -gain 1/1]
    .body.console insert end "$image <-- hi_filter($source)\n"
    .body.stack.list insert 0 $image
}

proc func_detail_filter {} {
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    set m(x) 3
    set m(y) 3
    set m(0,0) -1 ; set m(1,0) -2 ; set m(2,0) -1
    set m(0,1) -2 ; set m(1,1) 28 ; set m(2,1) -2
    set m(0,2) -1 ; set m(1,2) -2 ; set m(2,2) -1

    set image [image create ik_image -convolute $source -matrix m -gain 1/16]
    .body.console insert end "$image <-- hi_filter($source)\n"
    .body.stack.list insert 0 $image
}

# The M3 classical halftone mask is below. 19 shades of gray are
# supported in this mask, so the numbers are X13 and rearranged into a
# matrix.
#  .  .  . 10  .  .  .
#  .  .  2 13 18  .  .
#  .  4  3 14 15 16  .
# 10 12 11  9  7  8 10
#  . 18 17  6  1  2  .
#  .  . 16  5  4  .  .
#  .  .  . 10  .  .  .

set fh_M3(x) 6
set fh_M3(y) 6

set fh_M3(0,0) 0x7575
set fh_M3(1,0) 0x5b5b
set fh_M3(2,0) 0x6868
set fh_M3(3,0) 0x8282
set fh_M3(4,0) 0x9c9c
set fh_M3(5,0) 0x8f8f

set fh_M3(0,1) 0x4e4e
set fh_M3(1,1) 0x0d0d
set fh_M3(2,1) 0x1a1a
set fh_M3(3,1) 0xa9a9
set fh_M3(4,1) 0xeaea
set fh_M3(5,1) 0xdddd

set fh_M3(0,2) 0x4141
set fh_M3(1,2) 0x3434
set fh_M3(2,2) 0x2727
set fh_M3(3,2) 0xb6b6
set fh_M3(4,2) 0xc3c3
set fh_M3(5,2) 0xd0d0

set fh_M3(0,3) 0x8282
set fh_M3(1,3) 0x9c9c
set fh_M3(2,3) 0x8f8f
set fh_M3(3,3) 0x7575
set fh_M3(4,3) 0x5b5b
set fh_M3(5,3) 0x6868

set fh_M3(0,4) 0xa9a9
set fh_M3(1,4) 0xeaea
set fh_M3(2,4) 0xdddd
set fh_M3(3,4) 0x4e4e
set fh_M3(4,4) 0x0d0d
set fh_M3(5,4) 0x1a1a

set fh_M3(0,5) 0xb6b6
set fh_M3(1,5) 0xc3c3
set fh_M3(2,5) 0xd0d0
set fh_M3(3,5) 0x4141
set fh_M3(4,5) 0x3434
set fh_M3(5,5) 0x2727

proc func_halftone {matrix} {
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    global $matrix

    set image [image create ik_image -halftone $source -matrix $matrix]
    ik_stack_push $image "$image <-- Halftone($source, $matrix)\n"
}

proc func_plane {plane} {
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    set image [ik_unique_image_name "$source\[$plane]"]
    image create ik_image $image -crop $source -plane $plane
    ik_stack_push $image "$image <-- Plane($source, $plane)\n"
}

set func_plane_planes 0
proc func_plane_interact {} {
    global func_plane_planes

    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    ik_get_values {
	{func_plane_planes {Planes:} text}
    }

    set cmd ""
    set name ""
    set list ""
    foreach plane $func_plane_planes {
	lappend cmd -plane $plane
	lappend list $plane
	append name $plane
    }

    set image [ik_unique_image_name "$source\[$name]"]
    eval "image create ik_image \$image -crop $source $cmd"
    ik_stack_push $image "$image <-- Plane($source, $list)\n"
}

proc func_laminate {} {
    set src1 [ik_stack_peek 0]
    if {[llength $src1] == 0} { return }

    set src2 [ik_stack_peek 1]
    if {[llength $src2] == 0} { return }

    set image [image create ik_image -laminate $src1 $src2]
    ik_stack_push $image "$image <-- Laminate($src1, $src2)\n"
}


set func_quant_palette "256"
proc func_quant {} {
    global func_quant_palette
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    ik_get_values "{func_quant_palette {Target size:} integer}"

    set image [image create ik_image \
	       -quant $source \
	       -palette $func_quant_palette]
    ik_stack_push $image "$image <-- Quant($source, $func_quant_palette)\n"
}

proc func_bilinear {} {
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    set image [image create ik_image -bilinear $source]
    ik_stack_push $image "$image <-- Bilinear($source)\n"
}

proc func_replicate {x y} {
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    set image [image create ik_image -replicate $source -xrep $x -yrep $y]
    ik_stack_push $image "$image <-- Repl($source, $x x $y)\n"
}

set func_replicate_x 1
set func_replicate_y 1
proc func_replicate_interact {} {
    global func_replicate_x func_replicate_y

    ik_get_values {
	{func_replicate_x {Width direction:} integer}
	{func_replicate_y {Height direction:} integer}
    }

    func_replicate $func_replicate_x $func_replicate_y
}

proc func_round {} {
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    set image [image create ik_image -round $source]
    ik_stack_push $image "$image <-- Round($source)\n"
}

proc func_red332 {} {
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    set image [image create ik_image -red332 $source]
    ik_stack_push $image "$image <-- Reduce332($source)\n"
}

proc func_color_diffuse {} {
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    set thresh {
	{0x2020 0x4040 0x6060 0x8080 0xa0a0 0xc0c0 0xe0e0}
	{0x2020 0x4040 0x6060 0x8080 0xa0a0 0xc0c0 0xe0e0}
	{0x4040 0x8080 0xc0c0}
    }

    set image [image create ik_image -ndiffuse $source -fuzzy -thresh $thresh]
    ik_stack_push $image "$image <-- NDiffuse($source {8 8 4} -fuzzy)\n"
}

proc func_red1dmed {} {
    global red1dmed_counts
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    set image [image create ik_image -red1dmed $source]
    ik_stack_push $image "$image <-- Reduce 1d Median($source)\n"
}

proc func_rgb_to_y {} {
    set rgb2y_n(x) 3
    set rgb2y_n(y) 1
    set rgb2y_n(0,0) 3
    set rgb2y_n(1,0) 6
    set rgb2y_n(2,0) 1

    set rgb2y_d(x) 3
    set rgb2y_d(y) 1
    set rgb2y_d(0,0) 10
    set rgb2y_d(1,0) 10
    set rgb2y_d(2,0) 10

    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    set image [image create ik_image -linear $source \
	    -num rgb2y_n -denom rgb2y_d]
    ik_stack_push $image "$image <-- Y($source)\n"
}

set solid_image_width  256
set solid_image_height 256
set solid_image_color  0

proc func_solid {} {

    global solid_image_width solid_image_height solid_image_color
    set rc [ik_get_values {
	{solid_image_width  {Image width:}  integer}
	{solid_image_height {Image height:} integer}
	{solid_image_color  {Image color:}  text}
    } CANCEL ]

    if {$rc != "OK"} return

    set image [image create ik_image -solid \
	    -width $solid_image_width \
	    -height $solid_image_height \
	    -color $solid_image_color]

    ik_stack_push $image "$image <-- Solid($solid_image_width,\
	$solid_image_height, $solid_image_color)\n"
}

proc func_sqe {} {
    set src1 [ik_stack_peek 0]
    if {[llength $src1] == 0} { return }

    set src2 [ik_stack_peek 1]
    if {[llength $src2] == 0} { return }

    set image [image create ik_image -square-error $src1 $src2]
    ik_stack_push $image "$image <-- square-error($src1, $src2)\n"
}


set func_thresh_value 32767

proc func_threshold {} {
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    global func_thresh_value

    ik_get_values "{func_thresh_value {Threshold:} integer}"
    set image [image create ik_image \
	    -threshold $source \
	    -thresh $func_thresh_value]
    ik_stack_push $image "$image <-- Threshold($source, $func_thresh_value)\n"
}

set func_rgb_red 0xffff
set func_rgb_green 0
set func_rgb_blue 0

proc func_rgbproj {} {
    global func_rgb_red func_rgb_green func_rgb_blue
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    ik_get_values {
	{func_rgb_red {Red:} integer}
	{func_rgb_green {Green:} integer}
	{func_rgb_blue  {Blue:} integer}
    }

    set elaps [time {
	set image [image create ik_image -rgbmatch $source \
		-proj \
		-red $func_rgb_red \
		-green $func_rgb_green \
		-blue $func_rgb_blue]
    }]
    ik_stack_push $image "$image <-- rgbproj($source, red)\n" "$elaps\n"
}

proc func_rgbproj_white {} {
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    set elaps [time {
	set image [image create ik_image -rgbmatch $source \
		-proj \
		-red 0xffff \
		-green 0xffff \
		-blue 0xffff]
    }]
    ik_stack_push $image "$image <-- rgbproj($source, white)\n" "$elaps\n"
}

proc func_average_column {} {
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }
    set elaps [time {set image [image create ik_image -average $source]}]
    ik_stack_push $image "$image <-- rgbproj($source, white)\n" "$elaps\n"
}

proc func_rgbmatch {} {
    global func_rgb_red func_rgb_green func_rgb_blue
    set source [ik_stack_peek]
    if {[llength $source] == 0} { return }

    ik_get_values {
	{func_rgb_red {Red:} integer}
	{func_rgb_green {Green:} integer}
	{func_rgb_blue  {Blue:} integer}
    }

    set elaps [time {
	set image [image create ik_image -rgbmatch $source \
		-match \
		-red $func_rgb_red \
		-green $func_rgb_green \
		-blue $func_rgb_blue]
    }]

    ik_stack_push $image "$image <-- rgbmatch($source, red)\n" "$elaps\n"
}

ik_add_help .help_hyper "Hyper Median Palette Reduction" {This is a \
high-performance colormap reduction function. It works by grouping all \
the image pixels into "bins" with other pixels of similar color. The \
boundries of the bins are adjusted so that they all contain roughly \
the same number of pixels. Then, a good color is chosen to represent \
all the pixels in the bin. Thus, the number of colors in the resulting \
image can be selected by choosing the number of bins.}

ik_add_help .help_red1dmed "1d Median Tone Reduction" {This function \
reduces the palette of the image, one plane at a time. The one \
dimension in the name comes from the single plane that is considered \
during processing. The algorithm is suitable for gray images, but can \
distort the chrominance when applied to color images. The Hyper Median \
palette reduction is an N-dimensional extension that works well with \
color images.}


# $Log: ikit_func.tcl,v $
# Revision 1.37  1999/02/10 03:59:57  steve
#  Redo interface to take thresholds, not counts.
#
# Revision 1.36  1999/02/03 16:46:57  steve
#  Add the ndiffuse operation for collor
#  diffusion, Rewrite palette handling,
#  fix several file types to better use
#  the palette information, rewrite the
#  rendering fucntions to use palette
#  information and white noise error diffusion,
#  or otherwise be cleaner and faster,
#  Add "visual" support in the ikit script.
#
# Revision 1.35  1999/01/12 23:46:24  steve
#  Add general alpha composite function.
#
# Revision 1.34  1998/12/18 21:21:24  steve
#  Add the column average.
#
# Revision 1.33  1998/10/17 18:10:56  steve
#  Add propogation parameter.
#
# Revision 1.32  1998/10/13 16:37:00  steve
#  Add lcexp
#  Add bmask
#  Add a detail enhancement filter.
#
# Revision 1.31  1998/05/26 21:17:52  steve
#  Color to Gray
#
# Revision 1.30  1998/05/25 23:23:11  steve
#  Rearrange some functions.
#
# Revision 1.29  1998/05/23 05:27:12  steve
#  Add bilinear upscaling.
#
# Revision 1.28  1998/04/24 03:34:54  steve
#  Generalized planing.
#
# Revision 1.27  1998/04/21 19:53:13  steve
#  Add te plane laminate function.
#
# Revision 1.26  1998/04/19 00:38:12  steve
#  More planes supported.
#
# Revision 1.25  1998/03/19 06:22:42  steve
#  Add some help panels.
#
# Revision 1.24  1998/01/23 01:12:09  steve
#  Add a manual crop function
#
# Revision 1.23  1998/01/09 01:31:59  steve
#  Add 90degree rotations.
#
# Revision 1.22  1998/01/06 06:56:17  steve
#  Slightly better labels.
#
# Revision 1.21  1998/01/06 06:52:04  steve
#  More replication capabilities.
#
# Revision 1.20  1998/01/04 00:41:12  steve
#  Add solid image synthesys.
#
# Revision 1.19  1998/01/02 17:32:48  steve
#  Add FFT.
#
# Revision 1.18  1997/10/10 19:34:30  steve
#  Quantize asks for target size.
#
# Revision 1.17  1997/09/19 17:04:23  steve
#  Add color quantization function.
#
