#
# Copyright (c) 1997,1998 Picture Elements, Inc.
# Stephen Williams (steve@picturel.com)
#
#    This source code is free software; you can redistribute it
#    and/or modify it in source code form under the terms of the GNU
#    Library General Public License as published by the Free Software
#    Foundation; either version 2 of the License, or (at your option)
#    any later version. In order to redistribute the software in
#    binary form, you will need a Picture Elements Binary Software
#    License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Library General Public License for more details.
#
#    You should have received a copy of the GNU Library General Public
#    License along with this program; if not, write to the Free
#    Software Foundation, Inc.,
#    59 Temple Place - Suite 330
#    Boston, MA 02111-1307, USA
#
#    You should also have recieved a copy of the Picture Elements
#    Binary Software License offer along with the source. This offer
#    allows you to obtain the right to redistribute the software in
#    binary (compiled) form. If you have not received it, contact
#    Picture Elements, Inc.,
#    777 Panoramic Way
#    Berkeley, CA 94704.
#
#ident "$Id: ikit_rawbits.tcl,v 1.7 1999/01/06 05:15:25 steve Exp $"

# --
# Set up the PNG stuff, including all the global functions
# needed. Install hooks into the GUI as needed.
#



# --
# 

set rawbits_file_load ""
set rawbits_file_save ""
set rawbits_file_width 1024
set rawbits_file_height 1024
set rawbits_file_planes 1

set yrbbits(file_load) ""
set yrbbits(file_save) ""
set yrbbits(file_width) 1024
set yrbbits(file_height) 1024

proc rawbits_save {} {
    global rawbits_file_save
    set source [ik_stack_peek]
    if {[llength $source] == 0} return


    set rc [ik_get_values \
	    "{rawbits_file_save {RAW file to save:} filename-w}" \
	    CANCEL ]
    if {$rc != "OK"} return

    ik_store $source -rawbits $rawbits_file_save
    ik_console "Stored $source in RAW file $rawbits_file_save\n"
}

proc rawbits_load {} {
    global rawbits_file_load rawbits_file_width rawbits_file_height \
	    rawbits_file_planes

    set rc [ik_get_values {
	{rawbits_file_load {RAW file to load:} filename-r}
	{rawbits_file_width {Image width:} integer}
	{rawbits_file_height {Image height:} integer}
	{rawbits_file_planes {Image planes:} integer}
    } CANCEL ]

    if {$rc != "OK"} return


    set label [ik_unique_image_name [file tail $rawbits_file_load]]

    image create ik_image $label -rawbits $rawbits_file_load \
	    -width $rawbits_file_width \
	    -height $rawbits_file_height \
	    -planes $rawbits_file_planes

    ik_stack_push $label "$label <-- RAWBITS($rawbits_file_load)\n"
}


ik_add_saver  "RAW Direct" rawbits_save
ik_add_loader "RAW Direct" rawbits_load

proc yrbbits_save {} {
    global yrbbits
    set source [ik_stack_peek]
    if {[llength $source] == 0} return


    set rc [ik_get_values \
	    "{yrbbits(file_save) {RAW file to save:} filename-w}" \
	    CANCEL ]
    if {$rc != "OK"} return

    ik_store $source -yrb $yrbbits(file_save)
    ik_console "Stored $source in RAW YCrCb file $yrbbits(file_save)\n"
}

proc yrbbits_load {} {
    global yrbbits

    set rc [ik_get_values {
	{yrbbits(file_load) {RAW YCrCb file to load:} filename-r}
	{yrbbits(file_width) {Image width:} integer}
	{yrbbits(file_height) {Image height:} integer}
    } CANCEL ]

    if {$rc != "OK"} return


    set label [ik_unique_image_name [file tail $yrbbits(file_load)]]

    image create ik_image $label -yrb $yrbbits(file_load) \
	    -width $yrbbits(file_width) \
	    -height $yrbbits(file_height)

    ik_stack_push $label "$label <-- RAW YCrCb($yrbbits(file_load))\n"
}

ik_add_saver  "RAW YCrCb" yrbbits_save
ik_add_loader "RAW YCrCb" yrbbits_load


#
# $Log: ikit_rawbits.tcl,v $
# Revision 1.7  1999/01/06 05:15:25  steve
#  Add the JPEG-style YRB file format.
#
# Revision 1.6  1998/10/09 22:25:54  steve
#  Much more portable file selection.
#
# Revision 1.5  1998/04/19 00:38:34  steve
#  Rawbits supports n planes.
#
# Revision 1.4  1997/10/10 20:09:16  steve
#  Add cancel button for file load and store,
#  Allow double-click to display image.
#
# Revision 1.3  1997/08/23 00:49:45  steve
#  change rawbits parms to conform to convention.
#
# Revision 1.2  1997/08/22 18:23:55  steve
#  Add rawbits store capability.
#
# Revision 1.1  1997/08/20 21:16:52  steve
#  Add support for raw image files. (8bitgray)
#
#
